@@ -6,11 +6,11 @@
 //! transmit messages.
 
 pub(crate) mod net;
+pub(crate) mod pubsub;
+pub(crate) mod rpc;
 
 pub mod config;
 pub mod process;
-pub mod pubsub;
-pub mod rpc;
 
 pub use config::Config;
 pub use process::Process;
